// This is Free Software
// See the GNU General Public License @ http://www.gnu.org/copyleft/gpl.html
// for details about the terms and conditions for using and distributing
// this piece of software.
// This software comes with absolutely NO WARRANTY!
// (c) 2003 by Daniel Gruen <daniel_gruen@web.de>

#include "ddd.h"

#include <cmath>

// projects xyz-coords on xy
ddcoord xycoord(dddcoord a,float k) {
   ddcoord tmp;
   tmp.x = int(a.x+400) + (400 - int(a.x+400)) * (0.5-(1.0/((a.z+200.0)/k+1.0)));
   tmp.y = int(a.y+300) + (300 - int(a.y+300)) * (0.5-(1.0/((a.z+200.0)/k+1.0)));
   return tmp;
}

// turns xyz-coords around an axis
// special thanks to Mr. M. Deffner
dddcoord xzturn(dddcoord a , int W)
{
   float D=atan(a.x/a.z);             //   { Ausgangswinkel }
   float G=D+(pi/180)*W;             //    { Drehwinkel }
   if (a.z<0) G=G+(pi);
    float R=sqrt(a.z*a.z+a.x*a.x); //      { Radius }
   a.z=int(R*cos(G));
   a.x=int(R*sin(G));
   return a;
}

dddcoord yzturn(dddcoord a , int W)
{
   float D=atan(a.y/a.z);             //   { Ausgangswinkel }
   float G=D+(pi/180)*W;             //    { Drehwinkel }
   if (a.z<0) G=G+(pi);
   float R=sqrt(a.z*a.z+a.y*a.y);  //      { Radius }
   a.z=int(R*cos(G));
   a.y=int(R*sin(G));
   return a;
}

dddcoord turn(dddcoord a, int xang, int yang)
{
     a.x -= 400;
     a.z -= 50;
     a = xzturn(a, xang);
     a.x += 400;
     a.z += 240;

     a.y -= 300;
     a.z -= 250;
     a = yzturn(a, yang);
     a.y += 300;
     a.z += 440;
     return a;
}

double vabs(vektor a){
    return sqrt(a.x*a.x+a.y*a.y+a.z*a.z);
}

vektor normalize(vektor a,double l=1.0){
    double f = l / vabs(a);
    a.x *= f; a.y *= f; a.z *= f;
    return a;
}


double intersection(vektor ga, vektor gu, vektor ea, vektor eu, vektor ev)
{
	// a = Geradenverankerung; w = Geradenschritt
	// b = Ebenenverankerung;  u,v = Ebenengeradenschritte
	double M[3][4], BUF;
	int I,J;
	M[0][0]=eu.x;    M[0][1]=-(gu.x);  M[0][2]=ev.x;  M[0][3]=ga.x-ea.x;
	M[1][0]=eu.y;    M[1][1]=-(gu.y);  M[1][2]=ev.y;  M[1][3]=ga.y-ea.y;
	M[2][0]=eu.z;    M[2][1]=-(gu.z);  M[2][2]=ev.z;  M[2][3]=ga.z-ea.z;


	for(I=0; I<=2; I++)
     {
          if (M[I][0]!=0)
          {
             for(J=3; J>=0; J--)
             {
                 M[I][J]=M[I][J]/M[I][0];
             }
          }
     }

     /* 2,1 UND 3,1 = 0 */
     I=1;
     if (M[I][0]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[0][J];
         }
     }
     I=2;
     if (M[I][0]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[0][J];
         }
     }

     /* 2. SPALTE NORMIEREN */
     for(I=0; I<=2; I++)
     {
          if (M[I][1]!=0)
          {
             BUF = M[I][1];
             for(J=0; J<=3; J++)
             {
                 M[I][J]=M[I][J]/BUF;
             }
          }
     }


     /* 1,2 UND 3,2 = 0 */
     I=0;
     if (M[I][1]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[1][J];
         }
     }
     I=2;
     if (M[I][1]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[1][J];
         }
     }

     /* 3. SPALTE NORMIEREN */
     for(I=0; I<=2; I++)
     {
          if (M[I][2]!=0)
          {
             BUF = M[I][2];
             for(J=0; J<=3; J++)
             {
                 M[I][J]=M[I][J]/BUF;
             }
          }
     }

     /* 1,3 UND 2,3 = 0 */
     I=0;
     if (M[I][2]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[2][J];
         }
     }
     I=1;
     if (M[I][2]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[2][J];
         }
     }

     /* NOCHMAL NORMIEREN */

     if (M[0][0]!=0)
     {
          M[0][3]=M[0][3]/M[0][0];
          M[0][0]=1;
     }

     if (M[1][1]!=0)
     {
          M[1][3]=M[1][3]/M[1][1];
          M[1][1]=1;
     }

	 //vektor s; s.x = M[0][3]; s.y = M[1][3]; s.z = M[2][3];
	 //return s;
	 return M[1][3];
}

vektor vintersection(vektor ga, vektor gu, vektor ea, vektor eu, vektor ev)
{
	// ga = Geradenverankerung; gu = Geradenschritt
	// ea = Ebenenverankerung;  eu,ev = Ebenengeradenschritte
	double M[3][4], BUF;
	int I,J;
	M[0][0]=eu.x;    M[0][1]=-(gu.x);  M[0][2]=ev.x;  M[0][3]=ga.x-ea.x;
	M[1][0]=eu.y;    M[1][1]=-(gu.y);  M[1][2]=ev.y;  M[1][3]=ga.y-ea.y;
	M[2][0]=eu.z;    M[2][1]=-(gu.z);  M[2][2]=ev.z;  M[2][3]=ga.z-ea.z;


	for(I=0; I<=2; I++)
     {
          if (M[I][0]!=0)
          {
             for(J=3; J>=0; J--)
             {
                 M[I][J]=M[I][J]/M[I][0];
             }
          }
     }

     /* 2,1 UND 3,1 = 0 */
     I=1;
     if (M[I][0]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[0][J];
         }
     }
     I=2;
     if (M[I][0]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[0][J];
         }
     }

     /* 2. SPALTE NORMIEREN */
     for(I=0; I<=2; I++)
     {
          if (M[I][1]!=0)
          {
             BUF = M[I][1];
             for(J=0; J<=3; J++)
             {
                 M[I][J]=M[I][J]/BUF;
             }
          }
     }


     /* 1,2 UND 3,2 = 0 */
     I=0;
     if (M[I][1]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[1][J];
         }
     }
     I=2;
     if (M[I][1]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[1][J];
         }
     }

     /* 3. SPALTE NORMIEREN */
     for(I=0; I<=2; I++)
     {
          if (M[I][2]!=0)
          {
             BUF = M[I][2];
             for(J=0; J<=3; J++)
             {
                 M[I][J]=M[I][J]/BUF;
             }
          }
     }

     /* 1,3 UND 2,3 = 0 */
     I=0;
     if (M[I][2]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[2][J];
         }
     }
     I=1;
     if (M[I][2]!=0)
     {
         for(J=0; J<=3; J++)
         {
             M[I][J]=M[I][J]-M[2][J];
         }
     }

     /* NOCHMAL NORMIEREN */

     if (M[0][0]!=0)
     {
          M[0][3]=M[0][3]/M[0][0];
          M[0][0]=1;
     }

     if (M[1][1]!=0)
     {
          M[1][3]=M[1][3]/M[1][1];
          M[1][1]=1;
     }

	 vektor s;
	 s.x = M[1][3]; // Faktor fr gu
	 s.y = M[0][3]; // Faktor fr eu
	 s.z = M[2][3]; // Faktor fr ev
	 return s;
}
