// This is Free Software
// See the GNU General Public License @ http://www.gnu.org/copyleft/gpl.html
// for details about the terms and conditions for using and distributing
// this piece of software.
// This software comes with absolutely NO WARRANTY!
// (c) 2003 by Daniel Gruen <daniel_gruen@web.de>

#include <qpixmap.h>
#include <qhbox.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qtabdialog.h>

#include <cmath>
#include <ctime>
#include <cstdlib>
#include <iostream>
#include <iomanip>

using namespace std;

#include "qray.h"
#include "ddd.h"
#include "qrayconfig.h"
#include "qrayobject.h"
#include "qraylight.h"
#include "qrayview.h"

/*--------------------------------------------------------*/

QRay::QRay(QWidget *parent, const char *name) : QWidget(parent, name){
	cout << "QRay::QRay started..." << endl;
	light = 0;
	plain = 0;
	view = 0;
	vektor u,v,w;

	v.x = 0; v.y = 0; v.z=-5000;
	view = new QRayView(v);

	v.x = 5; v.y = 5; v.z=-300;
	light = new QRayLight(v,QColor(255,255,255),80.0);

	v.x = 0.0;
	v.y = 0.0;
	v.z = 1400.0;
	u.x = 0.6;
	u.y = 0.001;
	u.z = 0.401;
	w.x = 0.001;
	w.y = 1.0;
	w.z = 0.001;
	plain = new QRayPlain(v,QColor(100,250,200),u,w);


	v.x = 0.0;
	v.y = 0.0;
	v.z = 1000.0;
	u.x = 0.1;
	u.y = 1.0;
	u.z = 0.1;
	w.x = 0.6;
	w.y = 0.3;
	w.z = 0.1;
	plain->next = new QRayRhomboid3D(v,QColor(100,100,255),u,w,200,200,1200);

	setMinimumSize(300,200);
	setFocusPolicy(QWidget::StrongFocus);
	setFocus();

	options = new QRayConfig(light,plain,view,0,"Optionen");

	pix = new QPixmap(width(),height());
	pix->fill(QColor(0,0,0));
	QPainter p(this);
	p.drawPixmap(0,0,*pix);
}

// repaint
void QRay::paintView()
{
	delete pix;
	pix = new QPixmap(width(),height());
	view->paintView(pix,light,plain);
	QPainter p(this);
	p.drawPixmap(0,0,*pix);
}

void QRay::configure(){
	options->show();
}

void QRay::paintEvent(QPaintEvent *)
{
	QPainter p(this);
	p.drawPixmap(0,0,*pix);
}
/*--------------------------------------------------------*/

// constructor for the menubar
QRayWidget::QRayWidget(QWidget *parent, const char *name) : QVBox(parent, name){
	QVBox * menu;
	menu = new QVBox(this, "qray-Menu");
	QHBox *amenu, *bmenu;
	amenu = new QHBox(menu);
	bmenu = new QHBox(menu);

	QPushButton *pb, *rb, *qb;
	pb = new QPushButton("&Refresh",amenu);
	rb = new QPushButton("&Configure",amenu);
	qb = new QPushButton("E&xit",amenu);

	menu->setMaximumHeight(50);

	QRay * ray;
	ray = new QRay(this, "Ray");
	//ray->setPlainS(100);
	//ray->setPlainT(0);
	connect(pb, SIGNAL(clicked()), ray, SLOT(paintView()));
	connect(rb, SIGNAL(clicked()), ray, SLOT(configure()));
	connect(qb, SIGNAL(clicked()), qApp,SLOT(quit()));
}

