// This is Free Software
// See the GNU General Public License @ http://www.gnu.org/copyleft/gpl.html
// for details about the terms and conditions for using and distributing
// this piece of software.
// This software comes with absolutely NO WARRANTY!
// (c) 2003 by Daniel Gruen <daniel_gruen@web.de>

#include "qrayconfig.h"
#include "qraylight.h"
#include "qrayobject.h"
#include "qrayview.h"

#include <qspinbox.h>
#include <qdial.h>
#include <kcolordialog.h>
#include <kcolorbutton.h>
#include <qcolor.h>

#include <iostream>
#include <cmath>

using namespace std;

QRayConfig::QRayConfig(QRayLight *firstl, QRayObject *firsto, QRayView *dview, QWidget *parent, const char *name)
	: Optionen(parent,name)
{
	object= firsto;
	light = firstl;
    view = dview;

    cpos1->setValue(int(view->a.x));
    cpos2->setValue(int(view->a.y));
    cpos3->setValue(int(view->a.z));

    breit->setValue(int((atan(view->width*0.5)/pi180 + 90.0)/1.8));
    hoch->setValue(int((atan(view->width*0.5)/pi180 + 90.0)/1.8));

    tracedepth->setValue(view->depth);

    int h,s,v;
    light->c.hsv(&h,&s,&v);
    colorSelect->setValues(h,s);
    QColor t;
    t.setHsv(int(h),int(s),255);
    colorButton->setColor(t);

    lpos1->setValue(int(light->a.x));
    lpos2->setValue(int(light->a.y));
    lpos3->setValue(int(light->a.z));

    intensity->setValue(int(light->intensity(1000)/lf));

    connect( colorSelect, SIGNAL( valueChanged(int,int) ), this, SLOT( lightHSChanged(int,int) ) );
    connect( intensity, SIGNAL( valueChanged(int) ), this, SLOT( lightIntensityChanged() ) );
    connect( breit, SIGNAL( valueChanged(int) ), this, SLOT( cameraFieldChanged() ) );
    connect( hoch, SIGNAL( valueChanged(int) ), this, SLOT( cameraFieldChanged() ) );
    connect( lpos1, SIGNAL( valueChanged(int) ), this, SLOT( lightPosChanged() ) );
    connect( lpos2, SIGNAL( valueChanged(int) ), this, SLOT( lightPosChanged() ) );
    connect( lpos3, SIGNAL( valueChanged(int) ), this, SLOT( lightPosChanged() ) );
    connect( cpos1, SIGNAL( valueChanged(int) ), this, SLOT( cameraPosChanged() ) );
    connect( cpos2, SIGNAL( valueChanged(int) ), this, SLOT( cameraPosChanged() ) );
    connect( cpos3, SIGNAL( valueChanged(int) ), this, SLOT( cameraPosChanged() ) );
    connect( tracedepth, SIGNAL( valueChanged(int) ), this, SLOT( cameraDepthChanged() ) );
}


void QRayConfig::cameraPosChanged()
{
    view->a.x=cpos1->value();
    view->a.y=cpos2->value();
    view->a.z=cpos3->value();
}

void QRayConfig::cameraDepthChanged()
{
    view->depth=tracedepth->value();
}

void QRayConfig::cameraFieldChanged()
{
    view->width= tan(breit->value()*0.9*pi180)*2.0;
    view->height= tan(hoch->value()*0.9*pi180)*2.0;
	// somebody please explain me why this is correct, I really can't figure it out, but it works
}

void QRayConfig::lightHSChanged(int h,int s)
{
    QColor t;
    t.setHsv(h,s,255);
    light->c= t;
    colorButton->setColor(t);
}

void QRayConfig::lightPosChanged()
{
    light->a.x=lpos1->value();
    light->a.y=lpos2->value();
    light->a.z=lpos3->value();
}

void QRayConfig::lightIntensityChanged()
{
    light->intens=intensity->value();
}

