// This is Free Software
// See the GNU General Public License @ http://www.gnu.org/copyleft/gpl.html
// for details about the terms and conditions for using and distributing
// this piece of software.
// This software comes with absolutely NO WARRANTY!
// (c) 2003 by Daniel Gruen <daniel_gruen@web.de>

#include "qrayobject.h"
#include "qraylight.h"
#include <iostream>
#include <cmath>

using namespace std;

/*--------------------------------------------------------*/

QRayObject::QRayObject(vektor da, QColor dc)
{
	a=da; c=dc;
	s = 1.001; // Blickwinkel-Abhngigkeit der Helligkeit
	t = 0.3;   // Lichteinfallswinkel-Abh. der Helligkeit
	next = 0;
}

void QRayObject::setS(double ds) {s=ds;}

void QRayObject::setT(double dt) {t=dt;}

void QRayObject::setS(int ds) {setS(double(ds/100.0));}

void QRayObject::setT(int dt) {setT(double(dt/100.0));}

double QRayObject::getT() {return t;}

double QRayObject::getS() {return s;}

QRayObject::~QRayObject()
{
	if(next);
		delete next;
}

double QRayObject::intersects(vektor r, vektor ga)
{
	r = ga;
	cout << "QRayObject::intersects called... shouldn't happen" << endl;
	return -1.0;
}

QColor QRayObject::color(vektor r, double ip, QRayLight *firstl, QRayObject *firsto, vektor camera, int depth)
{
	cout << "QRayObject::color called... shouldn't happen" << endl;
	return QColor(255,255,255);
}

/*--------------------------------------------------------*/

QRayPlain::QRayPlain(vektor da, QColor dc, vektor du, vektor dv)
: QRayObject(da,dc)
{
	cout << "QRayPlain::QRayPlain called... " << endl;
	u = normalize(du,1.0); v = normalize(dv,1.0);
	// n: Normalvektor, senkrecht zur Ebene mit Kreuzprodukt
	n.x = u.y*v.z - u.z*v.y;
	n.y = u.z*v.x - u.x*v.z;
	n.z = u.x*v.y - u.y*v.x;
	n = normalize(n,1.0);
	cout << "QRayPlain::QRayPlain finished... " << endl;
}

double QRayPlain::intersects(vektor r, vektor ga)
{
	return intersection(ga,r,a,u,v);
}

QColor QRayPlain::color(vektor r, double ip, QRayLight *firstl, QRayObject *firsto, vektor camera, int depth)
{
	return pointColor(this, r, ip, firstl, firsto, camera, depth);
}

/*--------------------------------------------------------*/

QRayRhomboid::QRayRhomboid(vektor da, QColor dc, vektor du, vektor dv, double dur, double dvr)
  : QRayObject(da,dc)
{
	u = normalize(du,1.0); v = normalize(dv,1.0);
	// n: Normalvektor, senkrecht zur Ebene mit Kreuzprodukt
	n.x = u.y*v.z - u.z*v.y;
	n.y = u.z*v.x - u.x*v.z;
	n.z = u.x*v.y - u.y*v.x;
	n = normalize(n,1.0);
	ur=dur; vr=dvr;
}

double QRayRhomboid::intersects(vektor r, vektor ga)
{
	vektor iv = vintersection(ga,r,a,u,v);

	// jetzt wre es Zeit fr eine schne mathematische Lsung!
	// ...
	// gefunden!

	if(fabs(iv.y)>ur || fabs(iv.z)>vr){ // auerhalb des Parallelogramms
		//cout << "n";
		return -1.0;
	}
	//cout << "o";
	return iv.x;
}

QColor QRayRhomboid::color(vektor r, double ip, QRayLight *firstl, QRayObject *firsto, vektor camera, int depth)
{
	return pointColor(this, r, ip, firstl, firsto, camera, depth);
}

/*--------------------------------------------------------*/

QRayRhomboid3D::QRayRhomboid3D(vektor da, QColor dc, vektor du, vektor dv, double dur, double dvr, double ddp)
  : QRayRhomboid(da,dc,du,dv,dur,dvr)
{
	vektor nda;
	nda.x = da.x+ur*u.x+ddp*n.x;
	nda.y = da.y+ur*u.y+ddp*n.y;
	nda.z = da.z+ur*u.z+ddp*n.z;
	next = new QRayRhomboid(nda,dc,v,n,dvr,ddp);

	nda.x = da.x-ur*u.x+ddp*n.x;
	nda.y = da.y-ur*u.y+ddp*n.y;
	nda.z = da.z-ur*u.z+ddp*n.z;
	next->next = new QRayRhomboid(nda,dc,v,n,dvr,ddp);

	nda.x = da.x+vr*v.x+ddp*n.x;
	nda.y = da.y+vr*v.y+ddp*n.y;
	nda.z = da.z+vr*v.z+ddp*n.z;
	next->next->next = new QRayRhomboid(nda,dc,u,n,dur,ddp);

	nda.x = da.x-vr*v.x+ddp*n.x;
	nda.y = da.y-vr*v.y+ddp*n.y;
	nda.z = da.z-vr*v.z+ddp*n.z;
	next->next->next->next = new QRayRhomboid(nda,dc,u,n,dur,ddp);

	nda.x = da.x + n.x*ddp*2.0;
	nda.y = da.y + n.y*ddp*2.0;
	nda.z = da.z + n.z*ddp*2.0;
	next->next->next->next->next = new QRayRhomboid(nda,dc,du,dv,dur,dvr);
}

/*--------------------------------------------------------*/

QColor pointColor(QRayObject *self, vektor r, double ip, QRayLight *firstl, QRayObject *firsto, vektor camera, int depth)
{
	// CAUTION: N HAS TO BE SET CORRECTLY WHEN USING THIS FUNCTION

	//r = normalize(r,1.0);
	// Projektion von r auf n
	vektor p;
	p.x = (r.x*self->n.x+r.y*self->n.y+r.z*self->n.z)*self->n.x;
	p.y = (r.x*self->n.x+r.y*self->n.y+r.z*self->n.z)*self->n.y;
	p.z = (r.x*self->n.x+r.y*self->n.y+r.z*self->n.z)*self->n.z;
	// reflektierter Blick
	vektor x;
	x.x = -r.x + 2.0*p.x;
	x.y = -r.y + 2.0*p.y;
	x.z = -r.z + 2.0*p.z;
	x = normalize(x,1.0);

	double rhell = 0.0;
	double ghell = 0.0;
	double bhell = 0.0;

  while(firstl){
	// Strahl zur jeweiligen Lichtquelle
	vektor l;
	l.x = firstl->a.x - ip*r.x;
	l.y = firstl->a.y - ip*r.y;
	l.z = firstl->a.z - ip*r.z;
	// Entfernung zur jeweiligen Lichtquelle
	double dist = sqrt(l.x*l.x+l.y*l.y+l.z*l.z);
	l = normalize(l,1.0);
	// was ist mit der Entfernung zur Kamera? hier kommt sie
	vektor c;
	c.x = camera.x - ip*r.x;
	c.y = camera.y - ip*r.y;
	c.z = camera.z - ip*r.z;
	dist += sqrt(c.x*c.x+c.y*c.y+c.z*c.z);

	// sehen, ob dieser Strahl vorher andere Ebene trifft
	QRayObject *tfirsto = firsto;
	bool lit = 1; // not in the shadow?
	while(tfirsto)
	{
		if(tfirsto!=self)
		{
			// shadow intersection point
			double sip = tfirsto->intersects(r,firstl->a);
			double mip = self->intersects(r,firstl->a);
			if((0.0<sip && sip<mip)/* || (0.0>sip && sip>ip)*/) // if we are in the shadow
				lit = 0;
		}
		tfirsto=tfirsto->next;
	}

	if(lit){
		double gamma = acos(x.x*l.x+x.y*l.y+x.z*l.z); // Winkel zwischen "reflektiertem Blick" und einfallendem Lichtstrahl
		double beta  = acos(self->n.x*l.x+self->n.y*l.y+self->n.z*l.z); // Winkel zwischen Normalvektor und einfallendem Lichtstrahl
		if(beta>pi/2.0)
			beta = pi/2.0;
		double hell  = pow(self->s,-(gamma*gamma)) * pow(cos(beta),self->t) * firstl->intensity(dist);
		//cout << "s=" << self->s << " alpha=" << alpha << " t=" << self->t << " beta=" << beta << " dist=" << dist;
		rhell += hell * firstl->c.red()/255.0;
		ghell += hell * firstl->c.green()/255.0;
		bhell += hell * firstl->c.blue()/255.0;
	}
/*	else{
		cout << "shadow!" << endl;
	}*/
	firstl=firstl->next;
  }

	// todo:
	// indirektes Licht,
	// Transparenz

	int cr = maxint(double(self->c.red()) * rhell,255);
	int cg = maxint(double(self->c.green()) * ghell,255);
	int cb = maxint(double(self->c.blue()) * bhell,255);
	//cout << " RGB: " << cr << " " << cg << " " << cb << endl;
	return QColor(cr,cg,cb);
}

int maxint(double x, int max)
{
	if(x>max)
		return max;
	return int(x);
}

