// This is Free Software
// See the GNU General Public License @ http://www.gnu.org/copyleft/gpl.html
// for details about the terms and conditions for using and distributing
// this piece of software.
// This software comes with absolutely NO WARRANTY!
// (c) 2003 by Daniel Gruen <daniel_gruen@web.de>

#include "qrayview.h"
#include "qraylight.h"
#include "qrayobject.h"

#include <qpixmap.h>
#include <qpainter.h>

#include <cmath>
#include <iostream>

using namespace std;

QRayView::QRayView(vektor da)
{
	a = da;
	width = height = 1.0;
	depth = 1;
}

void QRayView::paintView(QPixmap *pix, const QRayLight *firstl, const QRayObject *firsto)
{
	// under construction
	pix->fill(Qt::black);
	QPainter p(pix);
	double i,j;
	for(i=pix->width()/-2.0; i<pix->width()/2.0; i++)
	{
		for(j=pix->height()/-2.0; j<pix->height()/2.0; j++)
		{
		// compute the viewing angle for this pixel
		double alpha, delta; // alpha to the left, delta up
		alpha = atan(width*i/pix->width()); // durch Faktoren Blickfeld erweitern/einschrnken
		delta = atan(height*j/pix->height());
		// compute a ray vektor with this angle
		vektor r;
		/*
		ar.x = sin(alpha);
		ar.y = 0.0;
		ar.z = cos(alpha);
		dr.x = 0.0;
		dr.y = sin(delta);
		dr.z = cos(delta);
		r = ar+dr;
		*/
		r.x = sin(alpha); r.y = sin(delta); r.z = cos(alpha) + cos(delta);
		r = normalize(r,1.0);
		// find the intersections of this ray with
		// objects and light sources
		double dip,ip; // intersection point = ip * r
		dip = ip = -1.0;
		QRayObject *iobject = 0;
		QRayObject *thiso=const_cast<QRayObject*>(firsto);
		int num=1;
		while(thiso){
			dip = thiso->intersects(r,a);
			if(dip>0.0 && (ip<0.0 || dip<ip))
			{
				//cout << num << " ip=" << ip << " dip=" << dip << endl;
				ip=dip; iobject = thiso;
			}
			thiso = thiso->next;
			num++;
		}
		// paint pixel
		if(iobject){
			p.setPen(iobject->color(r,ip,const_cast<QRayLight*>(firstl),const_cast<QRayObject*>(firsto),a,depth));
			// I originally wrote "dip" in the former line, this caused trouble and I was searching for the error for 2 hours;
			// it was just one letter too much, how frustrating
			p.drawPoint(int(i+pix->width()/2.0),int(j+pix->height()/2.0));
		}
		}
	}
}

